"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PresetsBlockTypeCompletionProvider = void 0;
const theme_check_common_1 = require("@shopify/theme-check-common");
const RequestContext_1 = require("../../RequestContext");
const utils_1 = require("../../utils");
const BlockTypeCompletionProvider_1 = require("./BlockTypeCompletionProvider");
/**
 * The PresetsBlockTypeCompletionProvider offers value completions of the
 * `presets.[](recursive .blocks.[].type)` keys inside section and theme block `{% schema %}` tags.
 *
 * @example
 * {% schema %}
 * {
 *   "presets": [
 *     {
 *       "blocks": [
 *         { "type": "█" },
 *       ]
 *     },
 *   ]
 * }
 * {% endschema %}
 */
class PresetsBlockTypeCompletionProvider {
    constructor(getThemeBlockNames, getThemeBlockSchema) {
        this.getThemeBlockNames = getThemeBlockNames;
        this.getThemeBlockSchema = getThemeBlockSchema;
        this.uriPatterns = [/^.*\/(sections|blocks)\/[^\/]*\.liquid$/];
    }
    async completeValue(context, path) {
        if (!(0, utils_1.fileMatch)(context.doc.uri, this.uriPatterns) ||
            !(0, RequestContext_1.isLiquidRequestContext)(context) ||
            !isPresetBlockPath(path)) {
            return [];
        }
        const { doc } = context;
        const schema = await doc.getSchema();
        if (!schema || (0, theme_check_common_1.isError)(schema.parsed) || !(0, BlockTypeCompletionProvider_1.isSectionOrBlockSchema)(schema)) {
            return [];
        }
        let parsedBlockSchema = schema.parsed;
        if (isNestedBlockPath(path)) {
            const parentBlockName = getParentBlockName(schema.parsed, path);
            if (!parentBlockName) {
                return [];
            }
            const parentBlockSchema = await this.getThemeBlockSchema(doc.uri, parentBlockName);
            if (!parentBlockSchema ||
                (0, theme_check_common_1.isError)(parentBlockSchema.parsed) ||
                !(0, BlockTypeCompletionProvider_1.isSectionOrBlockSchema)(parentBlockSchema)) {
                return [];
            }
            parsedBlockSchema = parentBlockSchema.parsed;
        }
        const blocks = parsedBlockSchema.blocks;
        const blockGroups = {
            themeBlocks: false,
            specificBlockNames: [],
        };
        blocks.forEach((block) => {
            if (block.type === '@theme') {
                blockGroups.themeBlocks = true;
            }
            else if (!block.type.startsWith('@')) {
                blockGroups.specificBlockNames.push(block.type);
            }
        });
        let blockNames = blockGroups.specificBlockNames;
        if (blockGroups.themeBlocks) {
            blockNames.push(...(await this.getThemeBlockNames(doc.uri, false)));
        }
        return (0, BlockTypeCompletionProvider_1.createBlockNameCompletionItems)(blockNames);
    }
}
exports.PresetsBlockTypeCompletionProvider = PresetsBlockTypeCompletionProvider;
// `blocks` can be nested within other `blocks`
// We need to ensure the last leg of the path is { "blocks": [{ "type": "█" }] }
function isPresetBlockPath(path) {
    return path.at(0) === 'presets' && path.at(-3) === 'blocks' && path.at(-1) === 'type';
}
function isNestedBlockPath(path) {
    return path.at(-5) === 'blocks' && path.at(-3) === 'blocks' && path.at(-1) === 'type';
}
function getParentBlockName(parsedSchema, path) {
    return (0, theme_check_common_1.deepGet)(parsedSchema, [...path.slice(0, -3), 'type']);
}
//# sourceMappingURL=PresetsBlockTypeCompletionProvider.js.map